#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Util.Config import config
from ..Eloquent.MysqlBuilder import MysqlBuilder
from ..Eloquent.MongoBuilder import MongoBuilder


class BuilderFactory(object):
    _config = None

    def make(self, model, alias=None):
        self.init_config(model.__basepath__)
        database = model.__database__ if hasattr(model, '__database__') and model.__database__ is not None else 'default'
        return self.create_builder(model, self.get_db_type(database), alias)

    def init_config(self, config_path: str) -> None:
        if self._config is None:
            self._config = config.set_basepath(config_path)

    def get_db_type(self, database) -> str:
        config = self._config.items(database)
        return config.get('DB_TYPE', None)

    def create_builder(self, model, db_type, alias):
        if db_type == 'mysql':
            return MysqlBuilder(model, alias)
        elif db_type == 'mongodb':
            return MongoBuilder(model)
        raise Exception('Unsupported driver {}'.format(db_type))


builderfactory = BuilderFactory()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Util.Config import config
from .MysqlConnection import MysqlConnection
from .MongoConnection import MongoConnection


class ConnectionFactory(object):
    _config = None

    def make(self, database, config_path):
        self.init_config(config_path)
        db_type = self.get_db_type(database)
        return self.create_connection(db_type, database)

    def init_config(self, config_path: str) -> None:
        if self._config is None:
            self._config = config.set_basepath(config_path)

    def get_db_type(self, database) -> str:
        config = self._config.items(database)
        return config.get('DB_TYPE', None)

    def create_connection(self, db_type, database: str):
        if db_type == 'mysql':
            return MysqlConnection.instance(database, self.parse_config(database))
        elif db_type == 'mongodb':
            return MongoConnection.instance(database, self.parse_config(database))
        raise Exception('Unsupported driver {}'.format(database))

    def parse_config(self, database):
        return self._config.items(database)


connfactory = ConnectionFactory()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""环境变量类"""

__author__ = ''

from configparser import ConfigParser
import os


class MyConfigParser(ConfigParser):
    """解决ConfigParser会将key名转成小写的问题"""

    def optionxform(self, str):
        return str


class Config(object):
    """Env主要是为了获取.env中的配置数据"""

    _base_path = None

    _instance = None

    _config = {}

    _parser = None

    # 单例模式
    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(Config, cls).__new__(cls, *args, **kw)
        return cls._instance

    def __call__(self, name, defaule=''):
        return self._config.get(name, defaule)

    def __getattr__(self, name):
        return self._config.get(name)

    def items(self, key):
        if self._config.get(key, None) is None:
            filepath = self._base_path + '.env'
            if not (os.path.isfile(filepath) and os.access(filepath, os.R_OK)):
                raise Exception('.env file not access exception')
            self.get_parser().read(filepath)
            try:
                self._config[key] = dict(self.get_parser().items(key))
            except Exception:
                raise Exception('{} not find in .env file'.format(key))
        return self._config[key]

    def get_parser(self):
        if self._parser is None:
            self._parser = MyConfigParser()
        return self._parser

    def set_basepath(self, path):
        if self._base_path is None:
            self._base_path = path
        return self


config = Config()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Log日志类"""

__author__ = ''

import logging
import datetime


class Logger(object):
    _instance = None

    _logger = None

    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(Logger, cls).__new__(cls, *args, **kw)
        return cls._instance

    def info(self, msg):
        self.get_logger().info(msg)

    def set_path(self, path):
        self._path = path
        return self

    def get_logger(self):
        if self._logger is None:
            self._logger = logging.getLogger("logging")
            self._logger.setLevel("INFO")
            formatter = logging.Formatter('%(asctime)s 【%(levelname)s】%(message)s')
            fh = logging.FileHandler(self._path + "{}.log".format(datetime.datetime.now().strftime("%Y%m%d")))
            ch = logging.StreamHandler()
            fh.setFormatter(formatter)
            ch.setFormatter(formatter)
            self._logger.addHandler(fh)
            self._logger.addHandler(ch)
        return self._logger


logger = Logger()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import collections
from pymysql.cursors import DictCursor
from simpysql.Util.Expression import expression as expr, Expression
from simpysql.Util.Response import Response
from .BaseBuilder import BaseBuilder
from simpysql.Util.Dynamic import Dynamic


class MysqlBuilder(BaseBuilder):
    operators = [
        '=', '<', '>', '<=', '>=', '<>', '!=',
        'like', 'like binary', 'not like', 'between', 'ilike',
        '&', '|', '^', '<<', '>>',
        'rlike', 'regexp', 'not regexp',
        '~', '~*', '!~', '!~*', 'similar to',
        'not similar to', 'not ilike', '~~*', '!~~*', 'in', 'not in', 'not between'
    ]

    def __init__(self, model, alias=None):
        self.__model__ = model
        self.__alias__ = alias
        self.__where__ = []
        self.__orwhere__ = []                           # orwhere处理逻辑
        self.__whereor__ = []                           # orwhere处理逻辑
        self.__select__ = []                            # 检索的字段
        self.__limit__ = None                           # 检索的数据条数
        self.__orderby__ = []                           # 排序字段
        self.__groupby__ = []                           # 排序字段
        self.__offset__ = None                          # offset
        self.__lock__ = None                            # lock
        self.__join__ = []                              # leftjoin
        self.__union__ = []                             # union & unionall
        self.__on__ = []                                # leftjoin
        self.__having__ = None                          # having
        self.__subquery__ = []                          # subquery

    def first(self):
        self.__limit__ = 1
        data = self.get()
        if data:
            return data.pop()
        return data

    def get(self):
        return [Dynamic(index) for index in self._get_connection().execute(self._compile_select(), DictCursor)]

    def lists(self, columns):
        return Response(self._get_connection().execute(self._compile_select(), DictCursor)).tolist(columns)

    def data(self):
        return Response(self._get_connection().execute(self._compile_select(), DictCursor)).data()

    def response(self):
        return Response(self._get_connection().execute(self._compile_select(), DictCursor))

    def max(self, column):
        if isinstance(column, str) and column in self.__model__.columns:
            self.__select__ = ['max({}) as aggregate'.format(column)]
            data = self.first()
            return data['aggregate'] if data else None
        raise Exception('param invalid in function max')

    def min(self, column):
        if isinstance(column, str) and column in self.__model__.columns:
            self.__select__ = ['min({}) as aggregate'.format(column)]
            data = self.first()
            return data['aggregate'] if data else None
        raise Exception('param invalid in function min')

    def avg(self, column):
        if isinstance(column, str) and column in self.__model__.columns:
            self.__select__ = ['avg({}) as aggregate'.format(column)]
            data = self.first()
            return data['aggregate'] if data else None
        raise Exception('param invalid in function avg')

    def sum(self, column):
        if isinstance(column, str) and column in self.__model__.columns:
            self.__select__ = ['sum({}) as aggregate'.format(column)]
            data = self.first()
            return data['aggregate'] if data else None
        raise Exception('param invalid in function sum')

    def count(self):
        self.__select__ = ['count(*) as aggregate']
        data = self.first()
        return data['aggregate'] if data else None

    def exist(self):
        return True if self.count() > 0 else False

    def update(self, data):
        if data and isinstance(data, dict):
            data = self._set_update_time(data)
            return self._get_connection().execute(self._compile_update(data))

    def increment(self, key, amount=1):
        if isinstance(amount, int) and amount > 0:
            data = collections.defaultdict(dict)
            data[key] = '{}+{}'.format(expr.format_column(key), str(amount))
            data = self._set_update_time(data)
            return self._get_connection().execute(self._compile_increment(data))

    def decrement(self, key, amount=1):
        if isinstance(amount, int) and amount > 0:
            data = collections.defaultdict(dict)
            data[key] = '{}-{}'.format(expr.format_column(key), str(amount))
            data = self._set_update_time(data)
            return self._get_connection().execute(self._compile_increment(data))

    def create(self, data):
        if data:
            if data and isinstance(data, dict):
                data = [data]
            data = self._set_create_time(data)
            self._get_connection().execute(self._compile_create(data))
        return self

    def insert(self, columns, data):
        self._get_connection().execute(self._compile_insert(columns, data))
        return self

    def lastid(self):
        data = self._get_connection().execute(self._compile_lastid())
        return data[0][0] if data and data[0] and data[0][0] else None

    def delete(self):
        return self._get_connection().execute(self._compile_delete())

    def take(self, number):
        if number <= 0:
            raise Exception('take number invalid')
        self.__limit__ = int(number)
        return self

    def select(self, *args):
        self.__select__ = self._format_columns(list(args))
        return self

    def groupby(self, *args):
        self.__groupby__ = self._format_columns(list(args))
        return self

    def offset(self, number):
        if number <= 0:
            raise Exception('offset number invalid')
        self.__offset__ = int(number)
        return self

    def tosql(self):
        return self._compile_select()

    def where(self, *args):
        length = args.__len__()
        if length == 1 and isinstance(args[0], dict):
            self.__where__.append(args[0])
        elif length == 2:
            self.__where__.append({args[0]: self._check_columns_value(args[1])})
        elif length == 3:
            if args[1] in self.operators:
                if args[1] == '=':
                    self.__where__.append({args[0]: self._check_columns_value(args[2])})
                else:
                    self.__where__.append((args[0], args[1], self._check_columns_value(args[2])))
            else:
                raise Exception('operator key world not found: "{}"'.format(args[1]))
        else:
            raise Exception('bad parameters in where function')
        return self

    def orwhere(self, *args):
        length = args.__len__()
        if length == 1 and isinstance(args[0], dict):
            self.__orwhere__.append(args[0])
        elif length == 1 and isinstance(args[0], list):
            self.__orwhere__.append(args[0])
        elif length == 2:
            self.__orwhere__.append({args[0]: args[1]})
        elif length == 3:
            if args[1] in self.operators:
                if args[1] == '=':
                    self.__orwhere__.append({args[0]: args[2]})
                else:
                    self.__orwhere__.append((args[0], args[1], args[2]))
            else:
                raise Exception('operator key world not found: "{}"'.format(args[1]))
        else:
            raise Exception('bad parameters in where function')
        return self

    def whereor(self, *args):
        length = args.__len__()
        if length == 1 and isinstance(args[0], list):
            self.__whereor__.append(args[0])
        else:
            raise Exception('bad parameters in where function')
        return self

    def orderby(self, column, direction='asc'):
        if direction.lower() == 'asc':
            self.__orderby__.append(expr.format_column(column))
        else:
            self.__orderby__.append(expr.format_column(column) + ' desc')
        return self

    def execute(self, sql):
        return self._get_connection().execute(sql)

    def having(self, *args):
        length = args.__len__()
        if length == 2:
            self.__having__ = ' having {} {} {}'.format(args[0], '=',  expr.format_string(args[1]))
        elif length == 3:
            self.__having__ = ' having {} {} {}'.format(args[0], args[1], expr.format_string(args[2]))
        else:
            raise Exception('invalid parameter in having function')
        return self

    def lock_for_update(self):
        self.__lock__ = ' for update'
        return self

    def lock_for_share(self):
        self.__lock__ = ' lock in share mode'
        return self

    def leftjoin(self, model):
        if not (isinstance(model, BaseBuilder)):
            raise TypeError('invalid parameter type in leftjoin')
        self.__join__.append(('left join', model))
        return self

    def rightjoin(self, model):
        if not (isinstance(model, BaseBuilder)):
            raise TypeError('invalid parameter type in rightjoin')
        self.__join__.append(('right join', model))
        return self

    def join(self, model):
        return self.innerjoin(model)

    def innerjoin(self, model):
        if not (isinstance(model, BaseBuilder)):
            raise TypeError('invalid parameter type in innerjoin')
        self.__join__.append(('inner join', model))
        return self

    def union(self, model):
        if not (isinstance(model, BaseBuilder)):
            raise TypeError('invalid parameter type in union')
        self.__union__.append(('union', model))
        return self

    def unionall(self, model):
        if not (isinstance(model, BaseBuilder)):
            raise TypeError('invalid parameter type in unionall')
        self.__union__.append(('union all', model))
        return self

    def on(self, *args):
        length = args.__len__()
        if length == 2:
            self.__on__.append((args[0], '=', args[1]))
        elif length == 3:
            self.__on__.append((args[0], args[1], args[2]))
        else:
            raise Exception('invalid parameter in on function')
        return self

    def subquery(self, model, alias='tmp'):
        self.__subquery__.append((alias, model))
        return self

    def _compile_select(self):
        if len(self.__select__) == 0:
            self.__select__.append('*')
        subsql = ''.join(
            [self._compile_where(), self._compile_whereor(), self._compile_orwhere(), self._compile_groupby(), self._compile_orderby(),
             self._compile_having(), self._compile_limit(), self._compile_offset(), self._compile_lock()])
        joinsql = ''.join(self._compile_leftjoin())
        returnsql = "select {} from {}{}{}".format(','.join(self.__select__), self._tablename(), joinsql, subsql)
        if self.__union__:
            return '{}'.format(returnsql) + self._compile_union()
        return returnsql

    def _compile_create(self, data):
        return "insert into {} {} values {}".format(self._tablename(), self._columnize(data[0]), self._valueize(data))

    def _compile_insert(self, columns, data):
        return "insert into {} {} values {}".format(self._tablename(), self._columnize(columns), ','.join([tuple(index).__str__() for index in data]))

    def _compile_update(self, data):
        return "update {} set {}{}".format(self._tablename(), ','.join(self._compile_dict(data)), self._compile_where())

    def _compile_increment(self, data):
        subsql = ','.join(['{}={}'.format(expr.format_column(index), value) for index, value in data.items()])
        return "update {} set {}{}".format(self._tablename(), subsql, self._compile_where())

    def _compile_delete(self):
        return 'delete from {}{}'.format(self._tablename(), self._compile_where())

    def _compile_lastid(self):
        return 'select last_insert_id() as lastid'

    def _columnize(self, columns):
        return tuple(columns).__str__().replace('\'', '`')

    def _valueize(self, data):
        return ','.join([tuple(index.values()).__str__() for index in data])

    def _compile_groupby(self):
        return '' if len(self.__groupby__) == 0 else ' group by ' + ','.join(self.__groupby__)

    def _compile_orderby(self):
        return '' if len(self.__orderby__) == 0 else ' order by ' + ','.join(self.__orderby__)

    def _compile_limit(self):
        return '' if self.__limit__ is None else ' limit {}'.format(self.__limit__)

    def _compile_offset(self):
        return '' if self.__offset__ is None else ' offset {}'.format(self.__offset__)

    def _compile_lock(self):
        return '' if self.__lock__ is None else self.__lock__

    def _compile_leftjoin(self):
        if self.__join__:
            return ' ' + ' '.join(['{} {} on {}'.format(index, value._tablename(), value._compile_on()) for (index, value) in
                 self.__join__])
        return ''

    def _compile_union(self):
        if self.__union__:
            return ' ' + ' '.join(['{} ({})'.format(index, value.tosql()) for (index, value) in self.__union__])
        return ''

    def _compile_on(self):
        sqlstr = ['{} {} {}'.format(index[0], index[1], index[2]) for index in self.__on__]
        return ' and '.join(sqlstr)

    def _compile_having(self):
        if self.__having__:
            return self.__having__
        return ''

    def _compile_where(self):
        if len(self.__where__) > 0:
            sqlstr = []
            for index in self.__where__:
                if isinstance(index, dict):
                    sqlstr.append(' and '.join(self._compile_dict(index)))
                elif isinstance(index, tuple):
                    sqlstr.append(self._compile_tuple(index))
            return ' where {}'.format(' and '.join(sqlstr))
        return ''

    def _compile_orwhere(self):
        if len(self.__orwhere__) > 0:
            sqlstr = []
            for index in self.__orwhere__:
                if isinstance(index, dict):
                    subsql = self._compile_dict(index)
                    if len(subsql) == 1:
                        sqlstr.append(subsql.pop())
                    else:
                        sqlstr.append('({})'.format(' and '.join(subsql)))
                elif isinstance(index, tuple):
                    sqlstr.append(self._compile_tuple(index))
                elif isinstance(index, list):
                    subsql = []
                    for items in index:
                        if len(items) == 2:
                            subsql.append(self._compile_keyvalue(items[0], items[1]))
                        if len(items) == 3:
                            subsql.append(self._compile_tuple((items[0], items[1], items[2])))
                    sqlstr.append('({})'.format(' and '.join(subsql)))
                else:
                    raise Exception('undefined query condition {}'.format(index.__str__()))
            if len(self.__where__) > 0:
                return ' or {}'.format(' or '.join(sqlstr))
            return ' where {}'.format(' or '.join(sqlstr))
        return ''

    def _compile_whereor(self):
        if len(self.__whereor__) > 0:
            sqlstr = []
            for index in self.__whereor__:
                subsql = []
                for item in index:
                    if isinstance(item, dict):
                        if len(item) == 1:
                            subsql.append(self._compile_dict(item).pop())
                        else:
                            subsql.append('(' + ' and '.join(self._compile_dict(item)) + ')')
                    elif isinstance(item, list):
                        if isinstance(item[0], str):
                            subsql.append(self._compile_tuple(tuple(item)))
                        else:
                            subsql.append(self._compile_lists(item))
                    elif isinstance(item, tuple):
                        subsql.append(self._compile_tuple(item))
                    else:
                        raise Exception('whereor param invalid')
                sqlstr.append(' or '.join(subsql))
            if len(self.__where__) > 0:
                return ' and ({})'.format(' or '.join(sqlstr))
            return ' where ({})'.format(' or '.join(sqlstr))
        return ''

    def _compile_dict(self, data):
        return ['{}={}'.format(expr.format_column(index), expr.format_string(value)) for index, value in data.items()]

    def _compile_tuple(self, data):
        if data[1] in ['in', 'not in']:
            return self._compile_in((data[0], data[1], data[2]))
        elif data[1] in ['between', 'not between']:
            return self._compile_between((data[0], data[1], data[2]))
        return '{} {} {}'.format(expr.format_column(data[0]), data[1], expr.format_string(data[2]))

    def _compile_in(self, data):
        return '{} {} {}'.format(expr.format_column(data[0]), data[1], expr.list_to_str(data[2]))

    def _compile_list(self, data):
        length = len(data)
        if length == 2:
            return self._compile_keyvalue(data[0], data[1])
        if length == 3:
            return self._compile_tuple((data[0], data[1], data[2]))

    def _compile_lists(self, data):
        return_data = []
        for index in data:
            if isinstance(index, list):
                return_data.append(self._compile_list(index))
            if isinstance(index, tuple):
                return_data.append(self._compile_tuple(index))
        return '(' + ' and '.join(return_data) + ')'

    def _compile_between(self, data):
        if not (len(data) == 3 and len(data[2]) == 2):
            raise Exception('between param invalid')
        return '{} {} {} and {}'.format(expr.format_column(data[0]), data[1], expr.format_string(data[2][0]),
                                                     expr.format_string(data[2][1]))

    def _compile_keyvalue(self, key, value):
        return '{}={}'.format(expr.format_column(key), expr.format_string(value))

    def _compile_subquery(self):
        subquery = []
        for index, value in self.__subquery__:
            if isinstance(value, str):
                subquery.append('{} as {}'.format(value, index))
            else:
                subquery.append('({}) as {}'.format(value.tosql(), index))
        return ','.join(subquery)

    def _get_connection(self):
        return self.connect(self.__model__)

    def _check_columns_value(self, value):
        if self.__subquery__ and len(self.__subquery__) >= 2 and isinstance(value, str):
            tmp = value.split('.')
            if len(tmp) == 2 and tmp[0] in self._get_subquery_alias():
                return Expression(value)
        return value

    def _get_subquery_alias(self):
        return [index for index, value in self.__subquery__]

    def database(self, name):
        self.__model__.__database__ = name
        return self

    def _tablename(self):
        if self.__subquery__:
            return self._compile_subquery()
        if self.__alias__ is None:
            return self.__model__.__tablename__
        return self.__model__.__tablename__ + ' as {}'.format(self.__alias__)

    def _format_columns(self, columns):
        return list(map(lambda index: expr.format_column(index), columns))

    def _set_create_time(self, data):
        currtime = self.__model__.fresh_timestamp()
        update_column = self.__model__.update_time_column()
        create_column = self.__model__.create_time_column()
        for index in data:
            if create_column and create_column not in index:
                index[create_column] = currtime
            if update_column and update_column not in index:
                index[update_column] = currtime
        return data

    def _set_update_time(self, data):
        currtime = self.__model__.fresh_timestamp()
        update_column = self.__model__.update_time_column()
        if update_column and update_column not in data:
            data[update_column] = currtime
        return data

    def transaction(self, callback):
        return self._get_connection().transaction(callback)

    def transaction_wrapper(self, callback):
        return self._get_connection().transaction_wrapper(callback)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Eloquent.BaseBuilder import BaseBuilder
import pymysql

class Expression(object):

    def __init__(self, name=None):
        self.__name = name

    def format_column(self, key):
        return key

    def format_string(self, key):
        if isinstance(key, Expression):
            return key.__name
        elif isinstance(key, str):
            return "'{}'".format(pymysql.escape_string(key))
        elif isinstance(key, BaseBuilder):
            return "({})".format(key.tosql())
        return key

    def list_to_str(self, data):
        if data and isinstance(data, list):
            return tuple(data).__str__()
        if data and isinstance(data, tuple):
            return data.__str__()
        elif isinstance(data, BaseBuilder):
            return "(" + data.tosql() + ")"
        else:
            raise Exception('param invalid')


expression = Expression()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from .Connection import Connection
from ..Util.Logger import logger
import pymongo


class MongoConnection(Connection):

    _connection = {}

    _instance = {}  # 一个库一个instance

    def __init__(self, database, config):
        if config.get('LOG_DIR', None) is not None:
            self._logger = logger.set_path(config.get('LOG_DIR', None))
        self._database = database
        self._config = config

    def get(self, builder):
        _select = dict(builder.__select__) if builder.__select__ else None
        print(builder.__where__)
        model = self.db(builder._tablename()).find(builder.__where__, _select).skip(builder.__offset__).limit(builder.__limit__)
        if builder.__orderby__:
            model = model.sort(builder.__orderby__)
        return list(model)

    def create(self, builder, data):
        return self.db(builder._tablename()).insert(data)

    def update(self, builder, data):
        return self.db(builder._tablename()).update_many(builder.__where__, data)

    def delete(self, builder):
        return self.db(builder._tablename()).delete_many(builder.__where__)

    def db(self, tablename):
        return self.connect()[tablename]

    def count(self, builder):
        return self.db(builder._tablename()).find(builder.__where__).count()

    def connect(self):
        if self._connection.get(self._database, None) is None:
            config = self.parse_config(self._config)
            if config['user'] and config['password'] and config['authmechanism']:
                client = pymongo.MongoClient(host=config['host'], port=config['port'], username=config['user'], password=config['password'], authMechanism=config['authmechanism'])
            else:
                client = pymongo.MongoClient(host=config['host'], port=config['port'])
            self._connection[self._database] = client[config['db']]
        return self._connection[self._database]

    @classmethod
    def instance(cls, database, config):
        if cls._instance.get(database, None) is None:
            cls._instance[database] = MongoConnection(database, config)
        return cls._instance.get(database, None)

    def parse_config(self, config):
        return {
            'host': config.get('DB_HOST', ''),
            'port': int(config.get('DB_PORT', '')),
            'user': config.get('DB_USER', None),
            'password': config.get('DB_PASSWORD', None),
            'db': config.get('DB_NAME', ''),
            'charset': config.get('DB_CHARSET', None),
            'authmechanism': config.get('DB_AUTHMECHANISM', None),
        }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Util.Expression import expression as expr
from ..Util.Response import Response
from .BaseBuilder import BaseBuilder
from simpysql.Util.Dynamic import Dynamic
import pymongo


class MongoBuilder(BaseBuilder):
    operators = [
        '=', '<', '>', '<=', '>=', '<>', '!=',
        'like', 'in', 'not in', 'not between', 'exist',
        'not like', 'ilike', 'not ilike', 'between', 'mod', 'all', 'size'
    ]
    operators_map = {
        '<': '$lt',
        '<=': '$lte',
        '>': '$gt',
        '>=': '$gte',
        '!=': '$ne',
        'in': '$in',
        'not in': '$nin',
        'like': '$regex',
        'ilike': '$regex',
        'not like': '$regex',
        'not ilike': '$regex',
        'exist': '$exist',
        'mod': '$mod',
        'all': '$all',
        'size': '$size',
    }

    def __init__(self, model, alias=None):
        self.__model__ = model
        self.__alias__ = alias
        self.__where__ = {}
        self.__select__ = {}  # 检索的字段
        self.__limit__ = 0  # 检索的数据条数
        self.__orderby__ = []  # 排序字段
        self.__groupby__ = []  # 排序字段
        self.__offset__ = 0    # offset
        self.__lock__ = None  # lock
        self.__join__ = []  # leftjoin
        self.__union__ = []  # union & unionall
        self.__on__ = []  # leftjoin
        self.__having__ = None  # having
        self.__subquery__ = None  # subquery

    def first(self):
        data = self.get()
        if data:
            return data.pop()
        return data

    def get(self):
        return [Dynamic(index) for index in self._get_connection().get(self)]

    def lists(self, columns):
        return Response(self.get()).tolist(columns)

    def data(self):
        return Response(self.get()).data()

    def response(self):
        return Response(self.get())

    def count(self):
        return self._get_connection().count(self)

    def update(self, data):
        if data and isinstance(data, dict):
            data = self._set_update_time(data)
            return self._get_connection().update(self, {'$set': data})
        return None

    def create(self, data):
        if data:
            if isinstance(data, dict):
                data = [data]
            data = self._set_create_time(data)
            return self._get_connection().create(self, data)
        return None

    def delete(self):
        return self._get_connection().delete(self)

    def take(self, number):
        if number <= 0:
            raise Exception('take number invalid')
        self.__limit__ = int(number)
        return self

    def select(self, *args):
        self.__select__.update({'_id': 0})
        [self.__select__.update({index: 1}) for index in args]
        return self

    def groupby(self, *args):
        self.__groupby__ = self._format_columns(list(args))
        return self

    def offset(self, number):
        if number <= 0:
            raise Exception('offset number invalid')
        self.__offset__ = int(number)
        return self

    def where(self, *args):
        length = args.__len__()
        if length == 1 and isinstance(args[0], dict):
            self.__where__.update(args[0])
        elif length == 2:
            if self.__where__.get('$and', None) is None:
                self.__where__['$and'] = []
            self.__where__['$and'].append({args[0]: args[1]})
        elif length == 3:
            if self.__where__.get('$and', None) is None:
                self.__where__['$and'] = []
            if args[1] in self.operators:
                if args[1] == '=':
                    self.__where__['$and'].append({args[0]: args[2]})
                else:
                    self.__where__['$and'].append(self._compile_tuple((args[0], args[1], args[2])))
            else:
                raise Exception('operator key world not found: "{}"'.format(args[1]))
        else:
            raise Exception('bad parameters in where function')
        return self

    def whereor(self, *args):
        if self.__where__.get('$or', None) is None:
            self.__where__['$or'] = []
        length = args.__len__()
        if length == 1:
            if isinstance(args[0], list):
                for index in args[0]:
                    self.__where__['$or'].append(self._compile(index))
            elif isinstance(args[0], dict):
                self.__where__['$or'].append(self._compile(args[0]))
        else:
            self.__where__['$or'].append(self._compile(args))
        return self

    def orderby(self, column, direction='asc'):
        if direction.lower() == 'asc':
            self.__orderby__.append((column, pymongo.ASCENDING))
        else:
            self.__orderby__.append((column, pymongo.DESCENDING))
        return self

    def sort(self, column, direction='asc'):
        return self.orderby(column, direction)

    def execute(self, sql):
        return self._get_connection().execute(sql)

    def _compile(self, data):
        length = data.__len__()
        if isinstance(data, dict):
            return data
        if length == 2:
            return {data[0]: data[1]}
        if length == 3:
            if data[1] in self.operators:
                if data[1] == '=':
                    return {data[0]: data[2]}
                else:
                    return {data[0]: {self.operators_map[data[1]]: data[2]}}
        raise Exception('bad parameters')

    def _compile_tuple(self, data):
        if data[1] == 'not like':
            return {data[0]: {self.operators_map[data[1]]: '^((?!{}).)*$'.format(data[2])}}
        elif data[1] == 'ilike':
            return {data[0]: {self.operators_map[data[1]]: data[2], '$options': 'i'}}
        elif data[1] == 'not ilike':
            return {data[0]: {self.operators_map[data[1]]: '^((?!{}).)*$'.format(data[2]), '$options': 'i'}}
        elif data[1] == 'between':
            if len(data[2]) != 2:
                raise Exception('between param error')
            return {data[0]: {'$lte': data[2][1], '$gte': data[2][0]}}
        elif data[1] == 'not between':
            if len(data[2]) != 2:
                raise Exception('not between param error')
            return {'$or': [{data[0]: {'$gt': data[2][1]}}, {data[0]: {'$lt': data[2][0]}}]}
        else:
            return {data[0]: {self.operators_map[data[1]]: data[2]}}

    def _get_connection(self):
        return self.connect(self.__model__)

    def database(self, name):
        self.__model__.__database__ = name
        return self

    def _tablename(self):
        if self.__subquery__:
            index, value = self.__subquery__
            return '({}) as {}'.format(value.tosql(), index)
        if self.__alias__ is None:
            return self.__model__.__tablename__
        return self.__model__.__tablename__ + ' as {}'.format(self.__alias__)

    def _format_columns(self, columns):
        return list(map(lambda index: expr.format_column(index), columns))

    def _set_create_time(self, data):
        currtime = self.__model__.fresh_timestamp()
        update_column = self.__model__.update_time_column()
        create_column = self.__model__.create_time_column()
        for index in data:
            if create_column and create_column not in index:
                index[create_column] = currtime
            if update_column and update_column not in index:
                index[update_column] = currtime
        return data

    def _set_update_time(self, data):
        currtime = self.__model__.fresh_timestamp()
        update_column = self.__model__.update_time_column()
        if update_column and update_column not in data:
            data[update_column] = currtime
        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from ..Util.Logger import logger


class Connection(object):

    _logger = None

    def log(self, sql):
        if self._logger:
            self._logger.info('【sql】:{}'.format(sql))

    def set_database(self, database):
        self._database = database



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""环境变量类"""

__author__ = ''


class Dynamic(dict):
    def __init__(self, d=None):
        if d is not None:
            for k, v in d.items():
                self[k] = v
        return super().__init__()

    def __key(self, key):
        return "" if key is None else key.lower()

    def __str__(self):
        import json
        return json.dumps(self)

    def __setattr__(self, key, value):
        self[self.__key(key)] = value

    def __getattr__(self, key):
        return self.get(self.__key(key))

    def __getitem__(self, key):
        return super().get(self.__key(key))

    def __setitem__(self, key, value):
        return super().__setitem__(self.__key(key), value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from .Eloquent.BuilderFactory import builderfactory
import time


class DBModel():
    __create_time__ = None  # 插入时间字段

    __update_time__ = None  # 更新时间字段

    # 获取创建时间字段
    def create_time_column(self):
        return self.__create_time__

    # 获取更新时间字段
    def update_time_column(self):
        return self.__update_time__

    # 获取当前时间戳
    def fresh_timestamp(self):
        return int(time.time())

    @classmethod
    def transaction(cls, callback):
        return cls.__new__(cls).transaction_wrapper(callback)

    def __new__(cls, *args, **kwargs):
        if len(args) > 0 and isinstance(args[0], str):
            return builderfactory.make(super(DBModel, cls).__new__(cls), alias=args[0])
        return builderfactory.make(super(DBModel, cls).__new__(cls))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Connections.ConnectionFactory import connfactory


class BaseBuilder(object):

    def connect(self, model):
        return connfactory.make(self.connection_name(model), model.__basepath__)

    def new_connect(self, model, database):
        return connfactory.make(database, model.__basepath__)

    def connection_name(self, model):
        return model.__database__ if hasattr(model, '__database__') and model.__database__ is not None else 'default'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .MysqlConnectionpool import connectionpool
from .Connection import Connection
from functools import wraps
from ..Util.Logger import logger


class MysqlConnection(Connection):
    _connection = {}

    _instance = {}  # 一个库一个instance

    def __init__(self, database, config):
        if config.get('LOG_DIR', None) is not None:
            self._logger = logger.set_path(config.get('LOG_DIR', None))
        self._database = database
        self._config = config

    def execute(self, sql, cursorclass=None):
        self.log(sql)
        cursor = self.connect().cursor(cursor=cursorclass)
        cursor.execute(sql)
        data = cursor.fetchall()
        if not hasattr(self, '_transaction'):
            self.connect().commit()
        cursor.close()
        return data

    def transaction(self, callback):
        try:
            self.start()
            result = callback()
            self.connect().commit()
            self.end()
            return result
        except Exception as e:
            self.connect().rollback()
            self.end()
            raise e

    def transaction_wrapper(self, callback):
        @wraps(callback)
        def wrapper(*args, **kwargs):
            try:
                self.start()
                result = callback(*args, **kwargs)
                self.connect().commit()
                self.end()
                return result
            except Exception as e:
                self.connect().rollback()
                self.end()
                raise e

        return wrapper

    def connect(self):
        if self._connection.get(self._database, None) is None:
            self._connection[self._database] = connectionpool.connection(self.parse_config(self._config),
                                                                         self._database)
        return self._connection[self._database]

    @classmethod
    def instance(cls, database, config):
        if cls._instance.get(database, None) is None:
            cls._instance[database] = MysqlConnection(database, config)
        return cls._instance.get(database, None)

    def start(self):
        self._transaction = True

    def end(self):
        del self._transaction

    def parse_config(self, config):
        return {
            'host': config.get('DB_HOST', ''),
            'port': int(config.get('DB_PORT', '')),
            'user': config.get('DB_USER', ''),
            'password': config.get('DB_PASSWORD', ''),
            'db': config.get('DB_NAME', ''),
            'charset': config.get('DB_CHARSET', ''),
        }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas


class Response(object):

    def __init__(self, data):
        self.__data__ = pandas.DataFrame(data) if data else None

    def data(self):
        return self.__data__

    def tolist(self, columns):
        if self.__data__ is not None:
            return self.__data__[columns].values.tolist()
        return []



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="simpysql",
    version="0.2.0",
    author="jeanku, liubing",
    author_email="",
    description="A simple mysql orm base on pymysql",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=["simpysql", "simpysql/Util", "simpysql/Eloquent", "simpysql/Connections"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pymysql',
        'pandas',
        'configparser',
        'DBUtils',
        'pymongo',
    ],
    entry_points={
        "console_scripts": ['simpysql = DBModel:DBModel']
    },
    keywords='MySQL ORM',
    python_requires='>=3',
)


from DBUtils.PooledDB import PooledDB, SharedDBConnection
import pymysql


class Connectionpool():
    _pool = {}

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Connectionpool, cls).__new__(cls, *args, **kwargs)
            return cls._instance

    def connection(self, pro_db_config, database):
        if self._pool.get(database, None) is None:
            self._pool[database] = PooledDB(
                creator=pymysql,  # 使用链接数据库的模块
                maxconnections=10,  # 连接池允许的最大连接数，0和None表示不限制连接数
                mincached=2,  # 初始化时，链接池中至少创建的空闲的链接，0表示不创建
                maxcached=5,  # 链接池中最多闲置的链接，0和None不限制
                maxshared=3,
                # 链接池中最多共享的链接数量，0和None表示全部共享。PS: 无用，因为pymysql和MySQLdb等模块的 threadsafety都为1，所有值无论设置为多少，_maxcached永远为0，所以永远是所有链接都共享。
                blocking=True,  # 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
                maxusage=None,  # 一个链接最多被重复使用的次数，None表示无限制
                setsession=[],  # 开始会话前执行的命令列表。如：["set datestyle to ...", "set time zone ..."]
                ping=0,
                **pro_db_config
            )

        return self._pool[database].connection()


connectionpool = Connectionpool()
